//  The Worldforge Project
//  Copyright (C) 1998, 1999 The Worldforge Project
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  For information about Worldforge and its authors, please contact
//  the Worldforge Web Site at http://www.worldforge.org.
//
// created by Karsten Laux 11/2000

#ifndef _JANUS_ELEMENT_GTK_H_
#define _JANUS_ELEMENT_GTK_H_

#if __GNUC__ < 3
#include <slist>
#else
#include <ext/slist>
#endif
#include <map>

#include "element.h"

namespace Janus {

/**Element Implementation interface.
 */
class ElementGtk : public Element
{
  friend class Element::Factory<ElementGtk>;  

 public: 
  /** Destructor.
  */
  virtual ~ElementGtk() {};
  
  
  /** call named method.
   */
  virtual void call(const std::string& name, const Value& val = VALUE_NONE)
    { Element::call(name, val); } ;

  ///
  void addReceiver(const std::string& signal, ElementGtk* element, 
		   const std::string& slot)
    {
      slots_[signal].push_front(std::pair<std::string, ElementGtk*>(slot, element));
    }

 protected:
  /** Constructor.
  */
  ElementGtk(const Parameters& param) : Element(param) {}; 
  ///
  typedef __gnu_cxx::slist< std::pair<std::string, ElementGtk*> > SlotList;
  ///
  std::map <std::string, SlotList > slots_;
  
  /** emit named event.
   */
  virtual void emit(const std::string& name, const Value& val = VALUE_NONE) 
    { 
      std::map<std::string, SlotList>::iterator itr = slots_.find(name);
      if(itr != slots_.end())
	{
	  SlotList::iterator ptr = itr->second.begin();
	  while(ptr != itr->second.end())
	    {
	      ptr->second->call(ptr->first, val);
	      ptr++;
	    }
	}
      else
	Element::call(name, val);
    }

};


} // namespace Janus

#endif
  
