//  The Worldforge Project
//  Copyright (C) 1998, 1999 The Worldforge Project
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  For information about Worldforge and its authors, please contact
//  the Worldforge Web Site at http://www.worldforge.org.
//
// created by Karsten Laux 11/2000

#ifndef _JANUS_ELEMENT_SIGC_H_
#define _JANUS_ELEMENT_SIGC_H_

#include "element.h"
#include <sigc++/signal_system.h>  


namespace Janus {

/**Element Implementation interface.
   abstract baseclass for libsigc++ implementation for signal/slot It only implements
   the event emitting function using a SigC::Signal instance.
 */
  class ElementSigC : public ::SigC::Object, public Element
{
  friend class Element::Factory<ElementSigC>;  

 public: 
  /** Destructor.
  */
  virtual ~ElementSigC() {};
  
  /** emit named event.
   */
  SigC::Signal2<void, const std::string&, const Value&> event;
  /** call named method.
   */
  virtual void call(const std::string& name, const Value& val = VALUE_NONE)
    { Element::call(name, val); } ;

 protected:
  /** Constructor.
  */
  ElementSigC(const Parameters& param) : Element(param) {}; 
  /** emit named event.
   */
  virtual void emit(const std::string& name, const Value& val = VALUE_NONE) 
    { event.emit(name, val); };
};


} // namespace Janus

#endif
  
