#include "listbox.h"
#include <uta/listbox.h>

using namespace Janus;

ListBox::ListBox(const Parameters& param) :
  WidgetBase(param)
{
  assert(parent_);

  uta::ListBox* listbox
    = new uta::ListBox(parent_, rect_.upperLeft().x, rect_.upperLeft().y,
		       rect_.width(), rect_.height());
  assert(listbox); 
  listbox->selectionChanged.connect(SigC::slot(this, &ListBox::slot_selectionChanged));

  widget_ = listbox;

  widget_->setAutoDelete(false);
  widget_->setColor(color_);

  if(hasAttr("background"))
    widget_->setBackground(background_);
  setValue(attributes_["value"]);
}

void 
ListBox::call(const std::string& name, const Value& val)
{
  // dynamic_cast<uta::Listbox*> returns non-valid pointers !!!
  uta::ListBox* listbox = (uta::ListBox*)widget_;
  
  if(name == "select")
    {
      std::cerr << "Janus::uta::ListBox::select not yet implemented ! " << std::endl;
    }
  else if(name == "addItem")
    {
      listbox->addElement(val);
    }
  else if(name == "clear")
    {
      listbox->clear();
    }
  else
    {
      WidgetBase::call(name, val);
    }
}

void
ListBox::setValue(const Value& val)
{
  //  uta::ListBox* listbox = (uta::ListBox*)widget_;
  std::cerr << "Janus::uta::ListBox::setValue not yet implemented ! " << std::endl;
}

Value 
ListBox::getValue(void) const
{
  uta::ListBox* listbox = (uta::ListBox*)widget_;
  return listbox->getSelection();
}

void
ListBox::slot_selectionChanged(const char* txt)
{
  ElementSigC::emit("selected", std::string(txt));
  ElementSigC::emit("selectionChanged", std::string(txt));
}

