#include "widgetbase.h"

#include <uta/application.h>
#include <uta/resources.h>

using namespace Janus;
  
WidgetBase::WidgetBase(const Parameters& param):
  ElementSigC(param),
  widget_(NULL),
  color_(uta::grey90),
  font_(NULL),
  background_(NULL)
{
  WidgetBase *parent = dynamic_cast<WidgetBase*>(param.parent);
  if(parent)
    parent_ = parent->widget_;
  else
    parent_ = uta::Application::Instance->rootWindow();

  update();
}

WidgetBase::~WidgetBase()
{
  if(widget_)
    delete widget_;
}


void
WidgetBase::update()
{
#ifndef NDEBUG
  std::cerr << "Janus::WidgetBase::update " << std::endl;
#endif  
  rect_ = uta::Rect(Element::getIntAttr("x",0),
		    Element::getIntAttr("y",0),
		    Element::getIntAttr("width",75),
		    Element::getIntAttr("height",25));

  std::string col = Element::getStringAttr("color","");
  if(RES_AVAILABLE(col))
    color_ = COLOR_RES(col);

  std::string font = Element::getStringAttr("font","");
  if(RES_AVAILABLE(font))
    font_ = FONT_RES(font);

  std::string back = Element::getStringAttr("background","");
  if(RES_AVAILABLE(back))
    background_ = SURFACE_RES(back);
}

void 
WidgetBase::setAttr(const std::string& id, const Value& attr)
{
#ifndef NDEBUG	
  std::cerr << "Janus::WidgetBase::setAttr " << std::endl;
#endif
  ElementSigC::setAttr(id, attr);
  update();
}

void 
WidgetBase::call(const std::string& name, const Value& val)
{
  // dynamic_cast<uta::Dialog*> returns non-valid pointers !!!
  uta::Widget* widget= (uta::Widget*)widget_;
  assert(widget);

  if(name == "show")
    {
      widget->show();
    }
  else if(name == "hide")
    {
      widget->hide();
    }
  else if(name == "enable")
    {
      widget->enable();
    }
  else if(name == "disable")
    {
      widget->disable();
    }
  else if(name == "resize")
    {
      int w,h;
      sscanf(val.c_str(),"%d %d",&w,&h);
      widget->resize(w,h);
    }
  else
    {
      ElementSigC::call(name, val);
    }
}
  

  

