# Configure paths for Janus
# stolen by Michael Koch
# Adapted by Al Riddoch
# stolen from Sam Lantinga of SDL
# stolen from Manish Singh
# stolen back from Frank Belew
# stolen from Manish Singh
# Shamelessly stolen from Owen Taylor

dnl AM_PATH_JANUS([MINIMUM-VERSION, [ACTION-IF-FOUND [, ACTION-IF-NOT-FOUND]]])
dnl Test for Janus, and define JANUS_CFLAGS and JANUS_LIBS
dnl
AC_DEFUN(AM_PATH_JANUS,
[dnl 
dnl Get the cflags and libraries from the janus-config script
dnl
AC_LANG_CPLUSPLUS
AC_ARG_WITH(janus-prefix,[  --with-janus-prefix=PREFIX
                          Prefix where Janus is installed (optional)],
            janus_prefix="$withval", janus_prefix="")
AC_ARG_WITH(janus-exec-prefix,[  --with-janus-exec-prefix=PREFIX
                          Exec prefix where Janus is installed (optional)],
            janus_exec_prefix="$withval", janus_exec_prefix="")
AC_ARG_ENABLE(janustest, [  --disable-janustest     Do not try to compile and run a test Janus program],
		    , enable_janustest=yes)

  if test x$janus_exec_prefix != x ; then
     janus_args="$janus_args --exec-prefix=$janus_exec_prefix"
     if test x${JANUS_CONFIG+set} != xset ; then
        JANUS_CONFIG=$janus_exec_prefix/bin/janus-config
     fi
  fi
  if test x$janus_prefix != x ; then
     janus_args="$janus_args --prefix=$janus_prefix"
     if test x${JANUS_CONFIG+set} != xset ; then
        JANUS_CONFIG=$janus_prefix/bin/janus-config
     fi
  fi

  AC_PATH_PROG(JANUS_CONFIG, janus-config, no)
  min_janus_version=ifelse([$1], ,0.3.11,$1)
  AC_MSG_CHECKING(for Janus - version >= $min_janus_version)
  no_janus=""
  if test "$JANUS_CONFIG" = "no" ; then
    no_janus=yes
  else
    JANUS_CFLAGS=`$JANUS_CONFIG $janusconf_args --cflags`
    JANUS_LIBS=`$JANUS_CONFIG $janusconf_args --libs`

    janus_major_version=`$JANUS_CONFIG $janus_args --version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\1/'`
    janus_minor_version=`$JANUS_CONFIG $janus_args --version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\2/'`
    janus_micro_version=`$JANUS_CONFIG $janus_config_args --version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\3/'`
    if test "x$enable_janustest" = "xyes" ; then
      ac_save_CFLAGS="$CFLAGS"
      ac_save_LIBS="$LIBS"
      CFLAGS="$CFLAGS $JANUS_CFLAGS"
      LIBS="$LIBS $JANUS_LIBS"
dnl
dnl Now check if the installed Janus is sufficiently new. (Also sanity
dnl checks the results of janus-config to some extent)
dnl
      rm -f conf.janustest
      AC_TRY_RUN([
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

char*
my_strdup (char *str)
{
  char *new_str;
  
  if (str)
    {
      new_str = (char *)malloc ((strlen (str) + 1) * sizeof(char));
      strcpy (new_str, str);
    }
  else
    new_str = NULL;
  
  return new_str;
}

int main (int argc, char *argv[])
{
  int major, minor, micro;
  char *tmp_version;

  /* This hangs on some systems (?)
  system ("touch conf.janustest");
  */
  { FILE *fp = fopen("conf.janustest", "a"); if ( fp ) fclose(fp); }

  /* HP/UX 9 (%@#!) writes to sscanf strings */
  tmp_version = my_strdup("$min_janus_version");
  if (sscanf(tmp_version, "%d.%d.%d", &major, &minor, &micro) != 3) {
     printf("%s, bad version string\n", "$min_janus_version");
     exit(1);
   }

   if (($janus_major_version > major) ||
      (($janus_major_version == major) && ($janus_minor_version > minor)) ||
      (($janus_major_version == major) && ($janus_minor_version == minor) && ($janus_micro_version >= micro)))
    {
      return 0;
    }
  else
    {
      printf("\n*** 'janus-config --version' returned %d.%d.%d, but the minimum version\n", $janus_major_version, $janus_minor_version, $janus_micro_version);
      printf("*** of Janus required is %d.%d.%d. If janus-config is correct, then it is\n", major, minor, micro);
      printf("*** best to upgrade to the required version.\n");
      printf("*** If janus-config was wrong, set the environment variable JANUS_CONFIG\n");
      printf("*** to point to the correct copy of janus-config, and remove the file\n");
      printf("*** config.cache before re-running configure\n");
      return 1;
    }
}

],, no_janus=yes,[echo $ac_n "cross compiling; assumed OK... $ac_c"])
       CFLAGS="$ac_save_CFLAGS"
       LIBS="$ac_save_LIBS"
     fi
  fi
  if test "x$no_janus" = x ; then
     AC_MSG_RESULT(yes)
     ifelse([$2], , :, [$2])     
  else
     AC_MSG_RESULT(no)
     if test "$JANUS_CONFIG" = "no" ; then
       echo "*** The janus-config script installed by Janus could not be found"
       echo "*** If Janus was installed in PREFIX, make sure PREFIX/bin is in"
       echo "*** your path, or set the JANUS_CONFIG environment variable to the"
       echo "*** full path to janus-config."
     else
       if test -f conf.janustest ; then
        :
       else
          echo "*** Could not run Janus test program, checking why..."
          CFLAGS="$CFLAGS $JANUS_CFLAGS"
          LIBS="$LIBS $JANUS_LIBS"
          AC_TRY_LINK([
#include <stdio.h>
],      [ return 0; ],
        [ echo "*** The test program compiled, but did not run. This usually means"
          echo "*** that the run-time linker is not finding Janus or finding the wrong"
          echo "*** version of Janus. If it is not finding Janus, you'll need to set your"
          echo "*** LD_LIBRARY_PATH environment variable, or edit /etc/ld.so.conf to point"
          echo "*** to the installed location  Also, make sure you have run ldconfig if that"
          echo "*** is required on your system"
	  echo "***"
          echo "*** If you have an old version installed, it is best to remove it, although"
          echo "*** you may also be able to get things to work by modifying LD_LIBRARY_PATH"],
        [ echo "*** The test program failed to compile or link. See the file config.log for the"
          echo "*** exact error that occured. This usually means Janus was incorrectly installed"
          echo "*** or that you have moved Janus since it was installed. In the latter case, you"
          echo "*** may want to edit the janus-config script: $JANUS_CONFIG" ])
          CFLAGS="$ac_save_CFLAGS"
          LIBS="$ac_save_LIBS"
       fi
     fi
     JANUS_CFLAGS=""
     JANUS_LIBS=""
     ifelse([$3], , :, [$3])
  fi
  AC_SUBST(JANUS_CFLAGS)
  AC_SUBST(JANUS_LIBS)
  rm -f conf.janustest
])
