//  The Worldforge Project
//  Copyright (C) 1998, 1999 The Worldforge Project
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  For information about Worldforge and its authors, please contact
//  the Worldforge Web Site at http://www.worldforge.org.
//
// created by Karsten Laux 11/2000

#include "element.h"

// for debugging statements
#ifndef NDEBUG
#include <iostream>

using std::cerr;
using std::endl;
#endif

using namespace Janus;

// static member ...
std::map<std::string, Janus::Factory<Element> * > *Element::factories_ = NULL;

Element::Element(const Element::Parameters& param) : 
    id_(param.id), 
    parent_(param.parent),
    attributes_(param.values),
    directory_(param.directory),
    cb_(NULL)
{
  if(!factories_)
    factories_ = new std::map<std::string, Janus::Factory<Element>*> ();
}

Element::~Element()
{
  directory_->erase(id_); 
}  
  
void 
Element::cleanup()
{
  if(factories_)
    delete factories_; 
}

void
Element::registerFactory(const std::string& type, 
				Janus::Factory<Element>* factory)
{
#ifndef NDEBUG
  cerr << "Janus::Element: registered element factory for type " 
       << "\"" << type << "\"" << endl;
#endif

  if(!factories_)
    factories_ = new std::map<std::string, Janus::Factory<Element>*> ();
  
  (*factories_)[type] = factory;
}

Element* 
Element::createElement(const std::string& type, const Element::Parameters& param)
{
  if(!factories_)
    return NULL;

  Element* element = NULL;
  std::map<std::string, Janus::Factory<Element>* >::iterator itr;


  //try to find factory for this type:
  itr = factories_->find(type);
  if(itr != factories_->end())
    {
#ifndef NDEBUG
      cerr << "Janus::Element: creating new element of type "<< type << endl;
#endif
      element = itr->second->New(param);
    }
  else
    {
#ifndef NDEBUG
      cerr << "Janus::Element: failed to find factory for " << type << endl;
#endif
    }

  return element;
}
