#include "engine.h"
#include "node.h"
#include <queue>
#include <iostream>

using namespace Janus;
using std::cerr;
using std::endl;

Engine::Engine()
{
}
  
Engine::~Engine()
{
  std::map<std::string, JanusEntry>::iterator itr = dialogs_.begin();
  std::vector<Connection*>::iterator citr;
  while(itr != dialogs_.end())
    {

      //      delete itr->second.dialog
      std::queue<Element*> toDel;
      Element::Directory::iterator ditr = itr->second.directory->begin();
      while(ditr != itr->second.directory->end())
	{
	  toDel.push(ditr->second);
	  ditr++;
	}
      while(!toDel.empty())
	{
	  delete toDel.front();
	  toDel.pop();
	}
      delete itr->second.directory;
      citr = itr->second.connections.begin();
      while(citr != itr->second.connections.end())
	{
	  if(*citr)
	    delete *citr;
	  citr++;
	}
      itr++;
    }

  Connection::cleanup();
  Element::cleanup();
}

std::string
Engine::create(Janus::Node* desc)
{
  if(desc->name != "janus")
    {
      cerr << "Janus::Engine::create() got an invalid janus description !" << endl;
      return "";
    }
    
  Janus::Node *structure=NULL, *style=NULL, *logic=NULL, *data=NULL;

  Janus::Node::ChildList::iterator itr = desc->childs.begin();
  while(itr != desc->childs.end())
    {  
      if(itr->name == "structure")
	structure = &(*itr);
      else if(itr->name == "style")
	style = &(*itr);
      else if(itr->name =="logic")
	logic = &(*itr);
      else if(itr->name == "data")
	data = &(*itr);
      else  
	{
	  cerr << "Janus::Engine::create() unknown toplevel tag "<<itr->name
	       << " will get ignored." << endl;
	}
	itr++;
    }
  
  if(!structure)
    {
      cerr<<"Janus::Engine::create() unable to read needed structure part "
	  <<"of description !" << endl;
      return "";
    } 
  
  JanusEntry entry;
  
  // the directory will belong to the new dialog
  entry.directory = new Element::Directory;

  // 1) process <data> part (if any)
  if(data)
    {
#ifndef NDEBUG	    
      cerr << "Janus::Engine::create() processing section <data> " << endl;
#endif      
      itr = data->childs.begin();
      while(itr != data->childs.end())
	{
	  std::string type = itr->name;
	  std::string part;
	  Element::Parameters param;
	  
	  Janus::Node::AttributeMap::iterator idPtr = itr->attributes.find("id");
	  if(idPtr != itr->attributes.end())
	    {
	      param.parent = NULL;
	      param.directory = entry.directory;
	      param.id = idPtr->second;
	      param.values = itr->attributes;
	      // ... hm what about the content of this node ?!
	      entry.data[idPtr->second] = Element::createElement(type, param);
	      
	    }
	  else
	    {
	      cerr << "ERROR: Janus::Engine::create() got data definition without ID." 
		   << endl;
	    }
	  itr++;
	}
    }
  
  // 2) process <style> description (may need stuff defined in <data>
  StyleSet styles;
  if(style)
    {
#ifndef NDEBUG
      cerr << "Janus::Engine::create() processing section <style> " << endl;
#endif
     
      itr = style->childs.begin();
      while(itr != style->childs.end())
	{
	  std::string partID;
	  Node::AttributeMap::iterator idPtr = itr->attributes.find("part");
	  if(idPtr != itr->attributes.end())
	    {
	      partID = idPtr->second;
	      idPtr = itr->attributes.begin();
	      while(idPtr != itr->attributes.end())
		{
		  // check for next existing attribute 
		  if(idPtr->first != "part")
		    styles.insert(std::pair<std::string, std::pair<std::string, Janus::Value> >(partID,std::pair<std::string, Janus::Value>(idPtr->first, idPtr->second)));
		  idPtr++;
		}
	    }
	  else
	    {
	      cerr << "ERROR: Janus::Engine::create() got style definition without part ID." 
		   << endl;
	    }
	  itr++;
	}
    }
  
  // 3) process <structure> which actually builds the dialog
  if(!structure)
    {
      cerr << "Janus::Engine::create() Error: description misses <structure> " 
	   << endl;
      return "";
    }

#ifndef NDEBUG
  cerr << "Janus::Engine::create() processing section <structure> " << endl;
#endif
  // recursively build element structure
  itr = structure->childs.begin();
  if(itr == structure->childs.end())
    return "";

  entry.dialog = createStructure(&(*itr), entry.directory, NULL, styles);
  
  if(!entry.dialog)
    return "";

  // 4) process <logic> to setup any connections
  if(logic)
    {
#ifndef NDEBUG
      cerr << "Janus::Engine::create() processing section <logic> " << endl;
#endif
      Connection::Parameters p;
      Janus::Node::AttributeMap::iterator attr;
      itr = logic->childs.begin();
      while(itr != logic->childs.end())
	{
#ifndef NDEBUG
	  cerr << "Janus::Engine::create() connection from ";
#endif
	  
	  attr = itr->attributes.find("sender");
	  if(attr != itr->attributes.end())
	    {
#ifndef NDEBUG
	      cerr << attr->second <<":";
#endif	      
	      Element::Directory::iterator ptr = entry.directory->find(attr->second);
	      if(ptr != entry.directory->end())
		p.creator = ptr->second;
	      else
		p.creator = NULL;
	    }
	  attr = itr->attributes.find("signal");
	  if(attr != itr->attributes.end())
	    {
#ifndef NDEBUG
	      cerr << attr->second << " to ";
#endif	      
	      p.signal = attr->second;
	    }
	  attr = itr->attributes.find("receiver");
	  if(attr != itr->attributes.end())
	    {	  
#ifndef NDEBUG
	      cerr << attr->second <<":";
#endif	      
	      Element::Directory::iterator ptr = entry.directory->find(attr->second);
	      if(ptr != entry.directory->end())
		p.consumer = ptr->second;
	      else
		p.consumer = NULL;
	    }
	  attr = itr->attributes.find("slot");
	  if(attr != itr->attributes.end())	
	    {
#ifndef NDEBUG
	      cerr << attr->second << endl;
#endif	      
	      p.slot = attr->second;
	    }
	  entry.connections.push_back(Connection::createConnection(p));
	  itr++;
	}
    }
  dialogs_[entry.dialog->id()] = entry;

  return entry.dialog->id();
}


Element*
Engine::createStructure(Janus::Node* desc, Element::Directory* directory,
			Element* parent, 
			const StyleSet& styles)
{
  if(!desc)
    return NULL;

  Element::Parameters param;
  param.parent = parent;
  param.directory = directory;

  std::string type = desc->name;

  // hmmm ... if not given, id will be "", is this OK ?
  Node::AttributeMap::iterator idPtr = desc->attributes.find("id");
  if(idPtr != desc->attributes.end())
    param.id = idPtr->second;
  
    
  std::pair<StyleSet::const_iterator, StyleSet::const_iterator> styleList;
  StyleSet::const_iterator stylePtr;

  // check for class styles
  styleList = styles.equal_range(type);
  stylePtr = styleList.first;
  while(stylePtr != styleList.second)
    {
      param.values[stylePtr->second.first] = stylePtr->second.second;
      stylePtr++;
    }

  // check for styles for specific element
  styleList = styles.equal_range(param.id);
  stylePtr = styleList.first;
  while(stylePtr != styleList.second)
    {
      param.values[stylePtr->second.first] = stylePtr->second.second;
      stylePtr++;
    }

  // finally set the given attributes (overwriting the style attributes)
  Node::AttributeMap::iterator attrItr = desc->attributes.begin();
  while(attrItr != desc->attributes.end())
    {
      param.values[attrItr->first] = attrItr->second;
      attrItr++;
    }
  
  Element* element = Element::createElement(type,param);
  if(!element)
    return NULL;

  Janus::Node::ChildList::iterator itr = desc->childs.begin();
  while(itr != desc->childs.end())
    {  
      createStructure(&(*itr), directory, element, styles);
      itr++;
    }

  return element;
}


void 
Engine::remove(const std::string& id)
{
  std::map<std::string, JanusEntry>::iterator itr = dialogs_.find(id);
  std::vector<Connection*>::iterator citr;
  if(itr != dialogs_.end())
    {
      //  delete itr->second.dialog;
      std::queue<Element*> toDel;
      Element::Directory::iterator ditr = itr->second.directory->begin();
      while(ditr != itr->second.directory->end())
	{
	  toDel.push(ditr->second);
	  ditr++;
	}
      while(!toDel.empty())
	{
#ifndef NDEBUG
	  cerr << "Janus::Engine deleting " << toDel.front()->id() << endl;
#endif	  
	  delete toDel.front();
	  toDel.pop();
	}
#ifndef NDEBUG
      cerr << "done." << endl;
#endif      
      delete itr->second.directory;
      citr = itr->second.connections.begin();
      while(citr != itr->second.connections.end())
	{
	  if(*citr)
	    delete *citr;
	  citr++;
	}
    }
  dialogs_.erase(id);
}

void
Engine::setAttributes(const std::string& id, const Values& vals)
{
  std::map<std::string, JanusEntry>::iterator itr = dialogs_.find(id);
  Values::const_iterator itr2 = vals.begin();
  if(itr != dialogs_.end())
    {
      while(itr2 != vals.end())
	{
	  itr->second.dialog->setAttr(itr2->first, itr2->second);
	  itr2++;
	}
    }
}

void
Engine::setAttributes(const std::string& id, const std::string& sub_id, const Values& vals)
{
  std::map<std::string, JanusEntry>::iterator itr = dialogs_.find(id);
  Values::const_iterator itr2 = vals.begin();
  if(itr != dialogs_.end())
    {	  
      Element::Directory::iterator itr3 = itr->second.directory->find(sub_id);
      if(itr3 != itr->second.directory->end())
	{
	  while(itr2 != vals.end())
	    {
	      itr3->second->setAttr(itr2->first, itr2->second);
	      itr2++;
	    }
	}
    }
}

Values 
Engine::getAttributes(const std::string& id, const std::string& sub_id)
{
  std::map<std::string, JanusEntry>::iterator itr = dialogs_.find(id);
  if(itr != dialogs_.end())
    {
      if(sub_id.empty())
	return itr->second.dialog->attributes();
      else
	{
	  Element::Directory::iterator itr2 = itr->second.directory->find(sub_id);
	  if(itr2 != itr->second.directory->end())
	    return itr2->second->attributes();
	}
    }
  return Values();
}

void
Engine::setValue(const std::string& id, const std::string& sub_id, const Value& val)
{
 std::map<std::string, JanusEntry>::iterator itr = dialogs_.find(id);
 if(itr != dialogs_.end())
   {
     Element::Directory::iterator itr2 = itr->second.directory->find(sub_id);
     if(itr2 != itr->second.directory->end())
       {
#ifndef NDEBUG
	 cerr << "Janus::Engine::setValue("<<id<<","<<sub_id<<","<<val<<")"<<endl;
#endif
	 itr2->second->setValue(val);
       }
     else
       cerr << "Janus::Engine::setValue(): could not find sub_id "<<sub_id << endl;
   }
 else
   cerr << "Janus::Engine::setValue(): could not find id "<<id << endl;
 
}

void
Engine::call(const std::string& id, const std::string& sub_id, 
	     const std::string& name,const Value& val)
{
 std::map<std::string, JanusEntry>::iterator itr = dialogs_.find(id);
 if(itr != dialogs_.end())
   {
     Element::Directory::iterator itr2 = itr->second.directory->find(sub_id);
     if(itr2 != itr->second.directory->end())
       {
#ifndef NDEBUG
	 cerr << "Janus::Engine::call("<<id<<","<<sub_id<<","<<name<<","<<val<<")"<<endl;
#endif
	 itr2->second->call(name, val);
       }
     else
       cerr << "Janus::Engine::call(): could not find sub_id "<<sub_id << endl;
   }
 else
   cerr << "Janus::Engine::call(): could not find id "<<id << endl;  
}

Values
Engine::getValues(const std::string& id)
{
 std::map<std::string, JanusEntry>::iterator itr = dialogs_.find(id);
 Values result;
 if(itr != dialogs_.end())
   {
     Element::Directory::iterator itr2 = itr->second.directory->begin();
     while(itr2 != itr->second.directory->end())
       {
	 result[itr2->first] = itr2->second->getValue();
	 itr2++;
       }
   }
 return result;
}

Values
Engine::execute(const std::string& id)
{
  std::map<std::string, JanusEntry>::iterator itr = dialogs_.find(id);
  if(itr != dialogs_.end())
    {
      itr->second.dialog->call("execute");

      return getValues(id);
    }
  return Values();
}

void
Engine::setCallback(const std::string& id, Callback* cb)
{
  std::map<std::string, JanusEntry>::iterator itr = dialogs_.find(id);
  if(itr != dialogs_.end())
    {
      itr->second.dialog->setCallback(cb);
    }
}

void
Engine::show(const std::string& id)
{
  std::map<std::string, JanusEntry>::iterator itr = dialogs_.find(id);
  if(itr != dialogs_.end())
    {
      itr->second.dialog->call("show");
    }
}

void
Engine::hide(const std::string& id)
{
  std::map<std::string, JanusEntry>::iterator itr = dialogs_.find(id);
  if(itr != dialogs_.end())
    {
      itr->second.dialog->call("hide");
    }
}
