//  The Worldforge Project
//  Copyright (C) 1998, 1999 The Worldforge Project
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  For information about Worldforge and its authors, please contact
//  the Worldforge Web Site at http://www.worldforge.org.
//
// created by Karsten Laux 11/2000

#ifndef _JANUS_ENGINE_H_
#define _JANUS_ENGINE_H_

#include <vector>

#include "node.h"
#include "element.h"
#include "connection.h"

namespace Janus {

/** Dialog Engine.
 *
 */
class Engine
{
 public:
  ///Constructor
  Engine();
  ///Destructor
  virtual ~Engine();

  /** Create a Dialog from a given description.
   *  \return an id identifying this dialog
   */
  virtual std::string create(Janus::Node* desc);

  /** Set attributes of the dialog widget.
   */
  void setAttributes(const std::string& id, const Values& vals);
  /** Set attributes of a single widget inside the given dialog.
   */
  void setAttributes(const std::string& id, const std::string& sub_id, const Values& vals);

  /** Get attributes of a single widget.
   */
  Values getAttributes(const std::string& id, const std::string& sub_id="");
  ///
  void setValue(const std::string& id, const std::string& sub_id, const Value& val);

  /** Get values of all widgets in given dialog.
   */
  Values getValues(const std::string& id);
 
  /** Delete Dialog <id>
   *  not necessary to call, because on shutdown all dialogs get deleted
   */
  virtual void remove(const std::string& id);

  /** Execute Dialog <id>.
   *  \return result of Dialog: a map of all widgets' values
   */
  virtual Values execute(const std::string& id);
  ///
  virtual void show(const std::string& id);
  ///
  virtual void hide(const std::string& id);
  ///
  void setCallback(const std::string& id, Callback*);
  /** Call method of specific widget.
      If the element cannot handle this call it is forwarded to
      the registered callback.
   */
  virtual void call(const std::string& id, const std::string& sub_id, 
	    const std::string& name,const Value& val);
 protected:  
  ///
  typedef std::multimap<std::string, std::pair<std::string, Janus::Value> > StyleSet;
  ///
  Element* createStructure(Janus::Node* desc, Element::Directory* directory,
			   Element* parent, 
			   const StyleSet& styles);
  ///
  typedef struct
  {
    // the actual dialog
    Element* dialog;
    // all involved connections (needed for cleaning up nicely)
    std::vector<Connection*> connections;
    // directory of all widgets of this dialog
    Element::Directory* directory;
    // all additional data, like bitmaps, fonts, strings
    std::map<std::string, Element*> data;
  } JanusEntry;

  ///
  std::map<std::string, JanusEntry> dialogs_;
};


} // namespace


#endif
